package nl.amis.ejb30.als;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;


public class LibraryService {
    public LibraryService() {
        EntityManagerFactory emf = 
            Persistence.createEntityManagerFactory("AlsPU");
        // create EntityManager
        EntityManager em = emf.createEntityManager();
        // note: alternative is EXTENDED EntityManager ; an EXTENDED EntityManager keeps objects attached
        // EntityManager em = emf.createEntityManager(PersistenceContextType.EXTENDED);
        this.setEntityManager(em);
    }

    private EntityManager _entityManager;

    public EntityManager getEntityManager() {
        return _entityManager;
    }

    public void setEntityManager(EntityManager entityManager) {
        _entityManager = entityManager;
    }

    public List<Publisher> findAllPublishers() {
        return getEntityManager().createQuery("select object(o) from Publisher o").getResultList();
    }

    public Publisher findPublisher(Long id) {
        return getEntityManager().find(Publisher.class, id);
    }

    public List<Author> findAllAuthors() {
        return getEntityManager().createQuery("select object(o) from Author o").getResultList();
    }
    public Author findAuthor(Long id) {
        return getEntityManager().find(Author.class, id);
    }
    
    public void modifyPublisher(Publisher publisher) {
        publisher = getEntityManager().merge(publisher);        
    }
    public void changePublisher(Publisher publisher) {
        EntityTransaction tx = getEntityManager().getTransaction();
        tx.begin();
        publisher = getEntityManager().merge(publisher);        
        // Commit the transaction
        tx.commit();
    }


    public void savePublisher(Publisher publisher) {
        EntityTransaction tx = getEntityManager().getTransaction();
        tx.begin();
        getEntityManager().persist(publisher);        
        // Commit the transaction
        tx.commit();
    }

    public void persistPublisher(Publisher publisher) {
        getEntityManager().persist(publisher);        
    }

    public Publisher createPublisher(Long id, String name) {
        Publisher p = new Publisher();
        p.setId(id);
        p.setName(name);
        savePublisher(p);        
        return p;
    }
    public void removePublisher(Publisher p) {
      // note: we cannot simply pass in any object to be removed:
      // the object may not be "detached"; any object that was created or
      // retrieved by the EntityManager in the current transaction is not
      // detached; any object from outside the current transaction however is
      // considered detached. We "attach"  a detached object by "merging" it:
      // note2: if we were using an EXTENDED EntityManager, any object that
      // was retrieved at any time by the EntityManager - across Transaction
      // boundaries - would still be "attached"
      
      // create an attached Publisher object for the (possibly) detached p
      Publisher p2 = getEntityManager().merge(p);
      // remove the attached counterpart of the (potentially) detached p
      getEntityManager().remove(p2);
    }
    public Object reattach(Object p) {
      // create an attached object for the (possibly) detached p
      Object p2 = getEntityManager().merge(p);
      return p2;
    }
    
    private EntityTransaction transaction;
    
    public void startTransaction() {
      transaction = getEntityManager().getTransaction();
      transaction.begin();        
    }
    
    public void endTransaction() {
       transaction.commit(); 
    }
}
