package nl.amis.ejb30.als;

import java.util.List;

import java.util.Vector;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;


public class LibraryService {
    public LibraryService() {
        EntityManagerFactory emf = 
            Persistence.createEntityManagerFactory("AlsPU");
        // create EntityManager
        EntityManager em = emf.createEntityManager();
        // note: alternative is EXTENDED EntityManager ; an EXTENDED EntityManager keeps objects attached
        // EntityManager em = emf.createEntityManager(PersistenceContextType.EXTENDED);
        this.setEntityManager(em);
    }

    private EntityManager _entityManager;

    public EntityManager getEntityManager() {
        return _entityManager;
    }

    public void setEntityManager(EntityManager entityManager) {
        _entityManager = entityManager;
    }

    public List<Publisher> findAllPublishers() {
        return getEntityManager().createQuery("select object(o) from Publisher o").getResultList();
    }

    public Publisher findPublisher(Long id) {
        return getEntityManager().find(Publisher.class, id);
    }

    public List<Author> findAllAuthors() {
        return getEntityManager().createQuery("select object(o) from Author o").getResultList();
    }

    public Author findAuthor(Long id) {
        return getEntityManager().find(Author.class, id);
    }
    public Book findBook(Long id) {
        return getEntityManager().find(Book.class, id);
    }

    public List<Book> findAllBooks() {
        return getEntityManager().createQuery("select object(o) from Book o").getResultList();
    }

    public List<Book> findAllJavaBooks() {
        return getEntityManager().createQuery("select object(o) from Book o where lower(o.title) like '%java%'") .getResultList();
    }

    public List<Book> findStevesBooks() {
        return getEntityManager().createQuery("SELECT b from Book AS b, in (b.authors) AS author where author.firstName ='Steve'").getResultList();
    }

    public List<Book> findJavaBooks() {
        return getEntityManager().createNamedQuery("javaBooks").getResultList();
    }


    public List<Book> findAllBooksOnSubject( String subject) {
        Query query = getEntityManager().createQuery("select object(o) from Book o where lower(o.title) like :subject") ;
        query.setParameter("subject", '%'+subject+'%');
        return query.getResultList();
    }

    public List<Book> findAllBooksOn( String subject) {
        Query q = getEntityManager().createNativeQuery("select * from als_books where regexp_like(title,'"+subject+"{1}','ci')", Book.class);
        return q.getResultList();
    }

    public List<Publisher> findBooklessPublishers() {
        return getEntityManager().createQuery("SELECT object(o) from Publisher o where o.bookCollection IS EMPTY").getResultList();
    }


    public List findJavaPublishers() {
        return getEntityManager().createQuery("SELECT object(p), object(authors)  from Publisher AS p, in (p.bookCollection) AS books, in (books.authors) authors , in (authors.books) books2"+
        " where lower(books2.title) like '%oracle%'  ").getResultList();
    }

    public List findSpecialPublishers() {
        return getEntityManager().createQuery("SELECT object(p), object(authors)  from Publisher AS p, in (p.bookCollection) AS books, in (books.authors) authors , in (authors.books) books2"+
        " where lower(books2.title) like '%oracle%'  ").getResultList();
    }

    public Object findBookSummary() {
        return getEntityManager().createQuery("SELECT max(b.publishYear), min(b.publishYear), count(b) from Book b").getSingleResult();
    }

    public List<Book> findRecentBooks() {
      return getEntityManager().createNativeQuery("select * from (select book.title, book.id, book.publish_year, book.pbr_id, row_number() over (partition by pbr_id order by publish_year, publish_month desc) rn from ALS_BOOKS book) where rn =1  ", nl.amis.ejb30.als.Book.class).getResultList();
    }

    public List<Author> findManAuthors() {
        return getEntityManager().createQuery("select object(o) from Author AS o JOIN o.books As b join b.publisher p where lower(p.website) like '%man%' ").getResultList();
    }
 
    


    
    public void modifyPublisher(Publisher publisher) {
        publisher = getEntityManager().merge(publisher);        
    }
    public void changePublisher(Publisher publisher) {
        EntityTransaction tx = getEntityManager().getTransaction();
        tx.begin();
        publisher = getEntityManager().merge(publisher);        
        // Commit the transaction
        tx.commit();
    }


    public void savePublisher(Publisher publisher) {
        EntityTransaction tx = getEntityManager().getTransaction();
        tx.begin();
        getEntityManager().persist(publisher);        
        // Commit the transaction
        tx.commit();
    }

    public void persistPublisher(Publisher publisher) {
        getEntityManager().persist(publisher);        
    }
    public void saveBook(Book book) {
        EntityTransaction tx = getEntityManager().getTransaction();
        tx.begin();
        getEntityManager().persist(book);        
        // Commit the transaction
        tx.commit();
    }

    public Publisher createPublisher(Long id, String name) {
        Publisher p = new Publisher();
        p.setId(id);
        p.setName(name);
        savePublisher(p);        
        return p;
    }
    public Book createBook(Long id, String title) {
        Book p = new Book();
        p.setId(id);
        p.setTitle(title);
        EntityTransaction tx = getEntityManager().getTransaction();
        tx.begin();
        getEntityManager().persist(p);        
        // Commit the transaction
        tx.commit();
        return p;
    }
    
    public void removePublisher(Publisher p) {
      // note: we cannot simply pass in any object to be removed:
      // the object may not be "detached"; any object that was created or
      // retrieved by the EntityManager in the current transaction is not
      // detached; any object from outside the current transaction however is
      // considered detached. We "attach"  a detached object by "merging" it:
      // note2: if we were using an EXTENDED EntityManager, any object that
      // was retrieved at any time by the EntityManager - across Transaction
      // boundaries - would still be "attached"
      
      // create an attached Publisher object for the (possibly) detached p
      Publisher p2 = getEntityManager().merge(p);
      // remove the attached counterpart of the (potentially) detached p
      getEntityManager().remove(p2);
    }
    public Object reattach(Object p) {
      // create an attached object for the (possibly) detached p
      Object p2 = getEntityManager().merge(p);
      return p2;
    }
    
    private EntityTransaction transaction;
    
    public void startTransaction() {
      transaction = getEntityManager().getTransaction();
      transaction.begin();        
    }
    
    public void endTransaction() {
       transaction.commit(); 
    }
}
